export declare function debug(str: string): void;
export declare type StateMachine = {
    nodes: Node[];
};
export declare type RunState = {
    candidateUsage: string | null;
    errorMessage: string | null;
    ignoreOptions: boolean;
    options: {
        name: string;
        value: any;
    }[];
    path: string[];
    positionals: {
        value: string;
        extra: boolean | typeof NoLimits;
    }[];
    remainder: string | null;
    selectedIndex: number | null;
};
export declare function makeStateMachine(): StateMachine;
export declare function makeAnyOfMachine(inputs: StateMachine[]): StateMachine;
export declare function injectNode(machine: StateMachine, node: Node): number;
export declare function simplifyMachine(input: StateMachine): void;
export declare function debugMachine(machine: StateMachine, { prefix }?: {
    prefix?: string;
}): void;
export declare function runMachineInternal(machine: StateMachine, input: string[], partial?: boolean): {
    node: number;
    state: RunState;
}[];
export declare function trimSmallerBranches(branches: {
    node: number;
    state: RunState;
}[]): {
    node: number;
    state: RunState;
}[];
export declare function selectBestState(input: string[], states: RunState[]): RunState;
export declare function aggregateHelpStates(states: RunState[]): RunState[];
declare type Transition = {
    to: number;
    reducer?: Callback<keyof typeof reducers, typeof reducers>;
};
declare type Node = {
    dynamics: [Callback<keyof typeof tests, typeof tests>, Transition][];
    shortcuts: Transition[];
    statics: {
        [segment: string]: Transition[];
    };
};
export declare function makeNode(): Node;
export declare function isTerminalNode(node: number): boolean;
export declare function cloneTransition(input: Transition, offset?: number): {
    to: number;
    reducer: "setCandidateUsage" | "setSelectedIndex" | "pushBatch" | "pushBound" | "pushPath" | "pushPositional" | "pushExtra" | "pushExtraNoLimits" | "pushTrue" | "pushFalse" | "pushUndefined" | "pushStringValue" | "setStringValue" | "inhibateOptions" | "useHelp" | "setError" | "setOptionArityError" | ["setCandidateUsage" | "setSelectedIndex" | "pushBatch" | "pushBound" | "pushPath" | "pushPositional" | "pushExtra" | "pushExtraNoLimits" | "pushTrue" | "pushFalse" | "pushUndefined" | "pushStringValue" | "setStringValue" | "inhibateOptions" | "useHelp" | "setError" | "setOptionArityError"] | ["setCandidateUsage" | "setSelectedIndex" | "pushBatch" | "pushBound" | "pushPath" | "pushPositional" | "pushExtra" | "pushExtraNoLimits" | "pushTrue" | "pushFalse" | "pushUndefined" | "pushStringValue" | "setStringValue" | "inhibateOptions" | "useHelp" | "setError" | "setOptionArityError", string] | ["setCandidateUsage" | "setSelectedIndex" | "pushBatch" | "pushBound" | "pushPath" | "pushPositional" | "pushExtra" | "pushExtraNoLimits" | "pushTrue" | "pushFalse" | "pushUndefined" | "pushStringValue" | "setStringValue" | "inhibateOptions" | "useHelp" | "setError" | "setOptionArityError", number] | ["setCandidateUsage" | "setSelectedIndex" | "pushBatch" | "pushBound" | "pushPath" | "pushPositional" | "pushExtra" | "pushExtraNoLimits" | "pushTrue" | "pushFalse" | "pushUndefined" | "pushStringValue" | "setStringValue" | "inhibateOptions" | "useHelp" | "setError" | "setOptionArityError", (string | undefined)?] | undefined;
};
export declare function cloneNode(input: Node, offset?: number): Node;
export declare function registerDynamic<T extends keyof typeof tests, R extends keyof typeof reducers>(machine: StateMachine, from: number, test: Callback<T, typeof tests>, to: number, reducer?: Callback<R, typeof reducers>): void;
export declare function registerShortcut<R extends keyof typeof reducers>(machine: StateMachine, from: number, to: number, reducer?: Callback<R, typeof reducers>): void;
export declare function registerStatic<R extends keyof typeof reducers>(machine: StateMachine, from: number, test: string, to: number, reducer?: Callback<R, typeof reducers>): void;
declare type UndefinedKeys<T> = {
    [P in keyof T]-?: undefined extends T[P] ? P : never;
}[keyof T];
declare type UndefinedTupleKeys<T extends unknown[]> = UndefinedKeys<Omit<T, keyof []>>;
declare type TupleKeys<T> = Exclude<keyof T, keyof []>;
export declare type CallbackFn<P extends any[], R> = (state: RunState, segment: string, ...args: P) => R;
export declare type CallbackFnParameters<T extends CallbackFn<any, any>> = T extends ((state: RunState, segment: string, ...args: infer P) => any) ? P : never;
export declare type CallbackStore<T extends string, R> = Record<T, CallbackFn<any, R>>;
export declare type Callback<T extends string, S extends CallbackStore<T, any>> = [
    TupleKeys<CallbackFnParameters<S[T]>>
] extends [UndefinedTupleKeys<CallbackFnParameters<S[T]>>] ? (T | [T, ...CallbackFnParameters<S[T]>]) : [T, ...CallbackFnParameters<S[T]>];
export declare function execute<T extends string, R, S extends CallbackStore<T, R>>(store: S, callback: Callback<T, S>, state: RunState, segment: string): R;
export declare function suggest(callback: Callback<keyof typeof tests, typeof tests>, state: RunState): string[] | null;
export declare const tests: {
    always: () => boolean;
    isOptionLike: (state: RunState, segment: string) => boolean;
    isNotOptionLike: (state: RunState, segment: string) => boolean;
    isOption: (state: RunState, segment: string, name: string, hidden?: boolean | undefined) => boolean;
    isBatchOption: (state: RunState, segment: string, names: string[]) => boolean;
    isBoundOption: (state: RunState, segment: string, names: string[], options: OptDefinition[]) => boolean;
    isNegatedOption: (state: RunState, segment: string, name: string) => boolean;
    isHelp: (state: RunState, segment: string) => boolean;
    isUnsupportedOption: (state: RunState, segment: string, names: string[]) => boolean;
    isInvalidOption: (state: RunState, segment: string) => boolean;
};
export declare const reducers: {
    setCandidateUsage: (state: RunState, segment: string, usage: string) => {
        candidateUsage: string;
        errorMessage: string | null;
        ignoreOptions: boolean;
        options: {
            name: string;
            value: any;
        }[];
        path: string[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    setSelectedIndex: (state: RunState, segment: string, index: number) => {
        selectedIndex: number;
        candidateUsage: string | null;
        errorMessage: string | null;
        ignoreOptions: boolean;
        options: {
            name: string;
            value: any;
        }[];
        path: string[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
    };
    pushBatch: (state: RunState, segment: string) => {
        options: {
            name: string;
            value: any;
        }[];
        candidateUsage: string | null;
        errorMessage: string | null;
        ignoreOptions: boolean;
        path: string[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    pushBound: (state: RunState, segment: string) => {
        options: {
            name: string;
            value: any;
        }[];
        candidateUsage: string | null;
        errorMessage: string | null;
        ignoreOptions: boolean;
        path: string[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    pushPath: (state: RunState, segment: string) => {
        path: string[];
        candidateUsage: string | null;
        errorMessage: string | null;
        ignoreOptions: boolean;
        options: {
            name: string;
            value: any;
        }[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    pushPositional: (state: RunState, segment: string) => {
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        candidateUsage: string | null;
        errorMessage: string | null;
        ignoreOptions: boolean;
        options: {
            name: string;
            value: any;
        }[];
        path: string[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    pushExtra: (state: RunState, segment: string) => {
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        candidateUsage: string | null;
        errorMessage: string | null;
        ignoreOptions: boolean;
        options: {
            name: string;
            value: any;
        }[];
        path: string[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    pushExtraNoLimits: (state: RunState, segment: string) => {
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        candidateUsage: string | null;
        errorMessage: string | null;
        ignoreOptions: boolean;
        options: {
            name: string;
            value: any;
        }[];
        path: string[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    pushTrue: (state: RunState, segment: string, name?: string) => {
        options: {
            name: string;
            value: any;
        }[];
        candidateUsage: string | null;
        errorMessage: string | null;
        ignoreOptions: boolean;
        path: string[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    pushFalse: (state: RunState, segment: string, name?: string) => {
        options: {
            name: string;
            value: any;
        }[];
        candidateUsage: string | null;
        errorMessage: string | null;
        ignoreOptions: boolean;
        path: string[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    pushUndefined: (state: RunState, segment: string) => {
        options: {
            name: string;
            value: any;
        }[];
        candidateUsage: string | null;
        errorMessage: string | null;
        ignoreOptions: boolean;
        path: string[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    pushStringValue: (state: RunState, segment: string) => {
        options: {
            name: string;
            value: any;
        }[];
        candidateUsage: string | null;
        errorMessage: string | null;
        ignoreOptions: boolean;
        path: string[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    setStringValue: (state: RunState, segment: string) => {
        options: {
            name: string;
            value: any;
        }[];
        candidateUsage: string | null;
        errorMessage: string | null;
        ignoreOptions: boolean;
        path: string[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    inhibateOptions: (state: RunState) => {
        ignoreOptions: boolean;
        candidateUsage: string | null;
        errorMessage: string | null;
        options: {
            name: string;
            value: any;
        }[];
        path: string[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    useHelp: (state: RunState, segment: string, command: number) => {
        options: {
            name: string;
            value: string;
        }[];
        candidateUsage: string | null;
        errorMessage: string | null;
        ignoreOptions: boolean;
        path: string[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    setError: (state: RunState, segment: string, errorMessage: string) => {
        errorMessage: string;
        candidateUsage: string | null;
        ignoreOptions: boolean;
        options: {
            name: string;
            value: any;
        }[];
        path: string[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
        selectedIndex: number | null;
    };
    setOptionArityError: (state: RunState, segment: string) => {
        errorMessage: string;
        candidateUsage: string | null;
        ignoreOptions: boolean;
        options: {
            name: string;
            value: any;
        }[];
        path: string[];
        positionals: {
            value: string;
            extra: boolean | typeof NoLimits;
        }[];
        remainder: string | null;
        selectedIndex: number | null;
    };
};
export declare const NoLimits: unique symbol;
export declare type ArityDefinition = {
    leading: string[];
    extra: string[] | typeof NoLimits;
    trailing: string[];
    proxy: boolean;
};
export declare type OptDefinition = {
    names: string[];
    description?: string;
    arity: number;
    hidden: boolean;
    allowBinding: boolean;
};
export declare class CommandBuilder<Context> {
    readonly cliIndex: number;
    readonly cliOpts: Readonly<CliOptions>;
    readonly allOptionNames: string[];
    readonly arity: ArityDefinition;
    readonly options: OptDefinition[];
    readonly paths: string[][];
    private context?;
    constructor(cliIndex: number, cliOpts: CliOptions);
    addPath(path: string[]): void;
    setArity({ leading, trailing, extra, proxy }: Partial<ArityDefinition>): void;
    addPositional({ name, required }?: {
        name?: string;
        required?: boolean;
    }): void;
    addRest({ name, required }?: {
        name?: string;
        required?: number;
    }): void;
    addProxy({ required }?: {
        name?: string;
        required?: number;
    }): void;
    addOption({ names, description, arity, hidden, allowBinding }: Partial<OptDefinition> & {
        names: string[];
    }): void;
    setContext(context: Context): void;
    usage({ detailed, inlineOptions }?: {
        detailed?: boolean;
        inlineOptions?: boolean;
    }): {
        usage: string;
        options: {
            definition: string;
            description: string;
        }[];
    };
    compile(): {
        machine: StateMachine;
        context: Context;
    };
    private registerOptions;
}
export declare type CliOptions = {
    binaryName: string;
};
export declare type CliBuilderCallback<Context> = (command: CommandBuilder<Context>) => CommandBuilder<Context> | void;
export declare class CliBuilder<Context> {
    private readonly opts;
    private readonly builders;
    static build<Context>(cbs: CliBuilderCallback<Context>[], opts?: Partial<CliOptions>): {
        machine: StateMachine;
        contexts: Context[];
        process: (input: string[]) => RunState;
        suggest: (input: string[], partial: boolean) => string[][];
    };
    constructor({ binaryName }?: Partial<CliOptions>);
    getBuilderByIndex(n: number): CommandBuilder<Context>;
    commands(cbs: CliBuilderCallback<Context>[]): this;
    command(): CommandBuilder<Context>;
    compile(): {
        machine: StateMachine;
        contexts: Context[];
        process: (input: string[]) => RunState;
        suggest: (input: string[], partial: boolean) => string[][];
    };
}
export {};
